﻿/* 

   o----------------------o----------------------o
           CAMERON McEFEE | 530.339.8372
       New Media Designer | cm@cameronmcefee.com
   o----------------------o----------------------o
                 Thanks for looking!
       Please email me with any bugs you find.
*/

// ---------------------------------------- Find Middle

function findMiddle(a) {
	if( app.documents.length > 0 ){
		// Get and set arguments for the funtion (CS4 single argument workaround)
		a = getArguments(a);
		var isHorizontal = a[0];
		var isVertical = a[1];
		var doc = app.activeDocument;
		var height = 0;
		var width = 0;
		var xPos = 0;
		var yPos = 0;
		var fromMarquee = false;
		var startRulerUnits = app.preferences.rulerUnits;
		var startDocResolution = doc.resolution;
		
		app.preferences.rulerUnits = Units.PIXELS;
		doc.resizeImage(undefined, undefined, 72, ResampleMethod.NONE );
		try {
			doc.selection.bounds;
			fromMarquee = true;
		} catch(err) {
		}
		if(fromMarquee == true){
			xPos = doc.selection.bounds[0];
			yPos = doc.selection.bounds[1];
			width = doc.selection.bounds[2]-doc.selection.bounds[0];
			height = doc.selection.bounds[3]-doc.selection.bounds[1];
		} else {
			height = doc.height;
			width = doc.width;
		}
		var horizontalPos = width/2+xPos;
		var verticalPos = height/2+yPos;
		if(isHorizontal=='true'){placeGuide(UnitValue(verticalPos, 'px'),'h');}
		if(isVertical=='true'){placeGuide(UnitValue(horizontalPos, 'px'),'v');}
		app.preferences.rulerUnits = startRulerUnits;
		doc.resizeImage(undefined, undefined, startDocResolution, ResampleMethod.NONE );
	}
}

// ---------------------------------------- Create Guides

function createGuides(a) {
	if( app.documents.length > 0 ){
		// Get and set arguments for the funtion (CS4 single argument workaround)
		a = getArguments(a);
		var tv = parseFloat(a[0]);
		var tu = a[1];
		var bv = parseFloat(a[2]);
		var bu = a[3];
		var lv = parseFloat(a[4]);
		var lu = a[5];
		var rv = parseFloat(a[6]);
		var ru = a[7];
		var cols = a[8];
		var cgv = parseFloat(a[9]);
		var cgu = a[10];
		var rows = a[11];
		var rgv = parseFloat(a[12]);
		var rgu = a[13];
		var doc = app.activeDocument;
		var fromMarquee = false;
		var width = 0;
		var height = 0;
		var xPos = 0;
		var yPos = 0;
		var insertPoint = 0;
		var colWidth = 0;
		var rowHeight = 0;
		var errors = '';
		var startRulerUnits = app.preferences.rulerUnits;
		var startDocResolution = doc.resolution;
		// Set up doc for processing measurements
		var rM = startDocResolution/72;
		doc.resizeImage(undefined, undefined, 72, ResampleMethod.NONE );
		app.preferences.rulerUnits = Units.PIXELS; 
		//Adjust for resolution. Not sure if I am missing something, or if Photoshop really ignores the resolution when converting units.
		if(tu!='px' && tu!='%'){tv*=rM;}
		if(bu!='px' && tu!='%'){bv*=rM;}
		if(lu!='px' && tu!='%'){lv*=rM;}
		if(ru!='px' && tu!='%'){rv*=rM;}
		if(cgu!='px' && tu!='%'){cgv*=rM;}
		if(rgu!='px' && tu!='%'){rgv*=rM;}
		
		var guideSpecs = {
							topMargin    : processInput(tv,tu,'h'),
							bottomMargin : processInput(bv,bu,'h'),
							leftMargin   : processInput(lv,lu,'h'),
							rightMargin  : processInput(rv,ru,'h'),
							colCount     : cols,
							colGutter    : processInput(cgv,cgu,'h'),
							rowCount     : rows, 
							rowGutter    : processInput(rgv,rgu,'h')
						};
					
		try {
			doc.selection.bounds;
			fromMarquee = true;
		} catch(err) {
		}
		if(fromMarquee == true){
			width = doc.selection.bounds[2]-doc.selection.bounds[0];
			height = doc.selection.bounds[3]-doc.selection.bounds[1];
			xPos = doc.selection.bounds[0];
			yPos = doc.selection.bounds[1];
		} else {
			width = doc.width;
			height = doc.height;
		}
		// Set Margins
		if(guideSpecs['topMargin']>0) {placeGuide(UnitValue(guideSpecs['topMargin']+yPos, 'px'),'h');}
		if(guideSpecs['bottomMargin']>0) {placeGuide(UnitValue((height-guideSpecs['bottomMargin'])+yPos, 'px'),'h');}
		if(guideSpecs['leftMargin']>0) {placeGuide(UnitValue(guideSpecs['leftMargin']+xPos, 'px'),'v');}
		if(guideSpecs['rightMargin']>0) {placeGuide(UnitValue((width-guideSpecs['rightMargin'])+xPos, 'px'),'v');}

		// Set columns
		if(guideSpecs['colCount']>0) {
			
			insertPoint = xPos;
			colWidth = (width-(guideSpecs['leftMargin']+guideSpecs['rightMargin']+((guideSpecs['colCount']-1)*guideSpecs['colGutter'])))/guideSpecs['colCount'];
			// Check to see if the columns are possible with the given measurements
			//alert(colWidth);
			if(colWidth>0 && (guideSpecs['leftMargin']+guideSpecs['rightMargin']) < width){
				// Place first line
				placeGuide(UnitValue(insertPoint, 'px'),'v');
				
				// Compensate for margins
				insertPoint += guideSpecs['leftMargin'];
				
				
				for(var i=1; i<guideSpecs['colCount']; i++){
					// Place column
					insertPoint += colWidth;
					placeGuide(UnitValue(insertPoint, 'px'),'v');
					
					// Place Gutter
					if(i<guideSpecs['colCount'] && guideSpecs['colGutter']>0){
						insertPoint += guideSpecs['colGutter'];
						placeGuide(UnitValue(insertPoint, 'px'),'v');
					}
				}
				
				// Compensate for margins
				insertPoint += colWidth;
				
				// Place end line
				insertPoint += guideSpecs['rightMargin'];
				placeGuide(UnitValue(insertPoint, 'px'),'v');
			} else {
				if(colWidth<0){errors += 'It is impossible to create columns with the specified measurements.\n\n';}
				if((guideSpecs['leftMargin']+guideSpecs['rightMargin']) > width){errors += 'It is impossible to create vertical margins with the specified measurements.\n\n';}
			}
		}

		// Set rows
		if(guideSpecs['rowCount']>0) {
			insertPoint = yPos;
			rowHeight = (height-(guideSpecs['topMargin']+guideSpecs['bottomMargin']+((guideSpecs['rowCount']-1)*guideSpecs['rowGutter'])))/guideSpecs['rowCount'];
			
			// Check to see if the columns are possible with the given measurements
			if(rowHeight>0 && (guideSpecs['topMargin']+guideSpecs['bottomMargin']) < height){
				
				// Place first line
				placeGuide(UnitValue(insertPoint, 'px'),'h');
				
				// Compensate for margins
				insertPoint += guideSpecs['topMargin'];

				for(var i=1; i<guideSpecs['rowCount']; i++){
					// Place column
					insertPoint += rowHeight;
					placeGuide(UnitValue(insertPoint, 'px'),'h');
					
					// Place Gutter
					if(i<guideSpecs['rowCount'] && guideSpecs['rowGutter']>0){
						insertPoint += guideSpecs['rowGutter'];
						placeGuide(UnitValue(insertPoint, 'px'),'h');
					}
				}
				
				// Compensate for margins
				insertPoint += rowHeight;
				
				
				// Place end line
				insertPoint += guideSpecs['bottomMargin'];
				placeGuide(UnitValue(insertPoint, 'px'),'h');
			} else {
				if(rowHeight<0){errors += 'It is impossible to create rows with the specified measurements.\n\n';}
				if((guideSpecs['topMargin']+guideSpecs['bottomMargin']) > height){errors += 'It is impossible to create the horizontal margins with the specified measurements.\n\n';}	
			}
		}
	
		if(errors != ''){
			errors = 'The following errors have occured:\n---------------------------------------\n\n' + errors;
			alert(errors);
		}

		// put original doc settings back
		app.preferences.rulerUnits = startRulerUnits;
		doc.resizeImage(undefined, undefined, startDocResolution, ResampleMethod.NONE );
	}
}


// ---------------------------------------- Misc

function trace(s){
	alert(String(s));
}


function processInput(value,units,orientation) {
	//The script below outputs a number that is the converted size.
	var doc = app.activeDocument;
	var fromMarquee = false;
	var width;
	var height;
	var returnValue;
	var startRulerUnits = app.preferences.rulerUnits;
	var startDocResolution = doc.resolution;
	// Set up doc for processing measurements
	doc.resizeImage(undefined, undefined, 72, ResampleMethod.NONE );
	app.preferences.rulerUnits = Units.PIXELS; 
	if(isNaN(value)) {
		returnValue = 0;
	} else if(units == 'null') {
		switch(String(app.preferences.rulerUnits)) {
			case 'Units.PIXELS':
				units = 'px';
				break;
			case 'Units.INCHES':
				units = 'inches';
				break;
			case 'Units.PICAS':
				units = 'picas';
				break;
			case 'Units.CM':
				units = 'cm';
				break;
			case 'Units.MM':
				units = 'mm';
				break;
			case 'Units.POINTS':
				units = 'points';
				break;
			case 'Units.PERCENT':
				units = 'percent';
				break;
			default:
				alert('Problem with blank unit value.');
				break;
		}
		returnValue = UnitValue(value, units).as('px');
	} else if(units == '%') { // If a percentage, do math.
		try {
			doc.selection.bounds;
			fromMarquee = true;
		} catch(err) {
		}
		if(fromMarquee == true){
			width = doc.selection.bounds[2]-doc.selection.bounds[0];
			height = doc.selection.bounds[3]-doc.selection.bounds[1];
		} else {
			width = doc.width;
			height = doc.height
		}
		if(orientation=='h') {
			var n = height*(value/100);
			returnValue = UnitValue(n).as('px');
		} else if (orientation=='v') {
			var n = width*(value/100);
			returnValue = UnitValue(n).as('px');
		} else {
			alert('Error: processInput() at orientation');
		}
	} else { // If not a percentage, do conversion.
		returnValue = UnitValue(value, units).as('px');
		
	}
	return(returnValue);
}

function placeGuide(value,orientation) {
	var idMk = charIDToTypeID( "Mk  " );
	var desc5 = new ActionDescriptor();
	var idNw = charIDToTypeID( "Nw  " );
	var desc6 = new ActionDescriptor();
	var idPstn = charIDToTypeID( "Pstn" );
	var idMeasure;
	if(app.preferences.rulerUnits=='Units.PIXELS'){ // pixels
		idMeasure = charIDToTypeID( "#Pxl" );
	} else if(app.preferences.rulerUnits=='Units.PERCENT') { // percent
		idMeasure = charIDToTypeID( "#Prc" );
	} else { // everything else
		idMeasure = charIDToTypeID( "#Rlt" );
	}
	
	desc6.putUnitDouble( idPstn, idMeasure, value );
	var idOrnt = charIDToTypeID( "Ornt" );
	var idOrnt = charIDToTypeID( "Ornt" );
	var idOrientation;
			if(orientation=='v'){
		idOrientation = charIDToTypeID( "Vrtc" );
	} else if (orientation=='h') {
		idOrientation = charIDToTypeID( "Hrzn" );
	}
	
	desc6.putEnumerated( idOrnt, idOrnt, idOrientation );
	var idGd = charIDToTypeID( "Gd  " );
	desc5.putObject( idNw, idGd, desc6 );
	executeAction( idMk, desc5, DialogModes.NO );
}

function getArguments(a) {
	a = a.split('|')
	return a;
}